;; VERSION March 7. 2017 ;;;;;;;;;;;;;;;;
;; Purpose:  extract 1D spectra from each slitlet and fit Gaussians to
;; the emission lines

;;; OUTPUT:  *.eps files contain the fitting results
;;;;;        * tables contain the output fitting parameters
;;;;;        * structures contain the input and output fitting parameters

;;; INPUT:  This pro is called by mosrunall*.pro;   

;;; First written in Jan-2014, bug report to T.-T. Yuan; yuantt83@gmail.com
;; Last modification Feb 2017 -------

;;; Version CHANGES: 1) this version takes into account of the varation in
;;; insturmental width across different bands, i also decide to take
;;; the instrumental width out after fitting instead of put it in the
;;; fitting, I then use the error propagation to derive the error in
;;; the instrinsic width

;; Version CHANGES: 2) this version allows user to fix the 1D extracting
;; aperture length as user defined



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;SUBROUTINES;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;; This section defines A bunch of user-defined functions for fitting emission lines.
;;;;; You can write your own functions to quatify the line
;;;;; profiles. For now I just use either a simple or double gaussians
;;;;; to fit all the emission lines.

;; wavelength in vaccumn please

pro HaNIIdoublet, x, par, f, pder
                                ; inswid=6.515837*1.d0 in K band  ;; inswid = instrument FWHM
   ;;; e.g., instrument FWHM = Gaussian_sigma*2.3548 !  # about 3pixels for 0.7" slit in MOSFIRE =6.5158371 Angs in Kband
;;; par 0-4:  z, Fha, sigma, FNII, cont
  ;;; we will subtract instrument width in the end
z = par[0]
Ha = (1+z)*6564.610d
NIIr = (1+z)*6585.23d
NIIb = (1+z)*6549.84d
Haf = par[1]
sigma = par[2]
NIIf = par[3]
c = par[4]
;sigma = sqrt((sigma^2)+(inswid/2.3548)^2)     ;;; sigma, instrument subtracted
sigma=sigma
f1 = (Haf/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-Ha)^2/sigma^2)
f2 = (NIIf/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-NIIr)^2/sigma^2)
f3 = (NIIf/3/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-NIIb)^2/sigma^2)
f =  c + f1 + f2 + f3
pder = fltarr(n_elements(x),n_elements(par))   ; no value returned.
end

 


pro double_OII, x, par, f, pder
;par =[z,f3727,f3729,sigma,cont]
 ; inswid=6.515837*1.d0  ;; inswid = instrument FWHM 
z = par[0] ;; par[5]
OII_3727 = (1+z)*0.372709*1.d4
OII_3729 = (1+z)*0.372988*1.d4
OII_3727f = par[1]
OII_3729f = par[2]
sigma = par[3]
c = par[4]
;sigma= sqrt((sigma^2)+(inswid/2.3548)^2) ; instrument profile convolve
f1 = (OII_3727f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-OII_3727)^2/sigma^2)
f2 = (OII_3729f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-OII_3729)^2/sigma^2)
f =  c + f1 + f2
pder = fltarr(n_elements(x),n_elements(par))   ; no value returned.
end


; -----------------------------------------------------------------------------
; --------------------------------------------------------------------------
pro singlet_O3, x, par, f, pder
;  inswid=6.515837*1.d0  ;; inswid = instrument FWHM 
;par =[z,flux,sigma,cont]
;;; This is the Gaussian Function
z = par[0]   ;redshift 
Ha = 5008.24*(1+z)  ;;;OII position 
Hainten = par[1]    ;; ha flux
sigma = par[2]   ;;; Gaussian sigma
;sigma= sqrt((sigma^2)+(inswid/2.3548)^2)  ; instrument profile convolve
Hasig=sigma
cont = par[3]   ;; continu level
f = cont + (Hainten/Hasig/sqrt(2.0*!pi)) * exp(-0.5*(x-Ha)^2/Hasig^2)   ;Gaussian Function
pder = fltarr(n_elements(x),n_elements(par))   ; no value returned.
end


pro double_O3, x, par, f, pder
;  inswid=6.515837*1.d0  ;; inswid = instrument FWHM 
;par =[z,flux,sigma,cont]
z = par[0] ;; par[5]
O3_4959 = (1+z)*0.496030*1.e4
O3_5007 = (1+z)*0.500824*1.e4
O3_5007f = par[1]
sigma = par[2]
c = par[3]
O3_4959f = par[1]/3.
;sigma= sqrt((sigma^2)+(inswid/2.3548)^2)  ; instrument profile convolve
f1 = (O3_5007f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-O3_5007)^2/sigma^2)
f2 = (O3_4959f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-O3_4959)^2/sigma^2)
f =  c + f1 + f2
pder = fltarr(n_elements(x),n_elements(par))   ; no value returned.
end

pro O3Hb, x, par, f, pder
;par =[z,FO3_5007,sigma,Fhb,cont]
;  inswid=6.515837*1.d0  ;; inswid = instrument FWHM 
z = par[0] ;; par[5]
O3_4959 = (1+z)*0.496030*1.e4
O3_5007 = (1+z)*0.500824*1.e4
Hb=(1+z)*4862.69
O3_5007f = par[1]
O3_4959f = par[1]/3.
sigma = par[2]
;sigma= sqrt((sigma^2)+(inswid/2.3548)^2)  ; instrument profile convolve
Hbf=par[3]
c = par[4]
f1 = (O3_5007f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-O3_5007)^2/sigma^2)
f2 = (O3_4959f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-O3_4959)^2/sigma^2)
f3 = (Hbf/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-Hb)^2/sigma^2)
f =  c + f1 + f2 + f3
pder = fltarr(n_elements(x),n_elements(par))   ; no value returned.
end



pro double_SII, x, par, f, pder
;par =[z,f6717,f6731,sigma,cont]
;  inswid=6.515837*1.d0  ;; inswid = instrument FWHM 
z = par[0] ;; par[5]
SII_6717 = (1+z)*0.671832*1.d4
SII_6731 = (1+z)*0.673271*1.d4
SII_6717f = par[1]
SII_6731f = par[2]
sigma = par[3]
c = par[4]
;sigma= sqrt((sigma^2)+(inswid/2.3548)^2)  ; instrument profile convolve
f1 = (SII_6717f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-SII_6717)^2/sigma^2)
f2 = (SII_6731f/sigma/sqrt(2.0*!pi)) * exp(-0.5*(x-SII_6731)^2/sigma^2)
f =  c + f1 + f2
pder = fltarr(n_elements(x),n_elements(par))   ; no value returned.
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;MAIN PROGRAM;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



pro fit1dmos_flux,datapath,pathdat,outputlist,intab,objlist_fc,objlist_std_fc,textpos,tablename,ps=ps,first=first,bandJ=bandJ,bandH=bandH,bandK=bandK,bandY=bandY,interact=interact
;fit1dmos_flux,datapath,pathdat,maskname,intab,objlist_fc,objlist_std_fc,/first
;;textpos='eps_H_S'

;;; extract 1d spectrum, and fit the profile
device, decomposed=0
loadcolors
!p.multi=[1,2,2]
;; if defined(bandJ) then begin
;;;

;;; CHECK & CHANGE THIS ACCORDINGLY: 
inswidK=6.515837*1.d0  ;; MOSFIRE instrument width in K band, 0.7" slit, 3pixels
inswidH=4.8868779*1.d0  ;; MOSFIRE instrument width in H band, 0.7" slit  , 3pixels
inswidJ=3.9095022*1.d0  ;; MOSFIRE instrument width in J band, 0.7" slit  , 3pixels
inswidY=3.2579187*1.d0  ;; MOSFIRE instrument width in Z band, 0.7" slit  , 3pixels

   IF defined(bandK) then inswid=inswidK
   IF defined(bandJ) then inswid=inswidJ
   IF defined(bandH) then inswid=inswidH
   IF defined(bandY) then inswid=inswidY


if keyword_set(interact) then begin
;readcol,pathdat+intab,lineguess,decx,flag,idm,objn,zp,rax,slitno,format='(f,a,i,i,a,f,a,i)'
readcol,pathdat+intab,lineguess,decx,flag,idm,objn,zp,rax,slitno,ymanguess,format='(f,a,i,i,a,f,a,i,f)'
endif else begin
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; INPUT FORMAT Supply your own;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NEW 2017 ;;; add ymanwidth and ymanfix2 to be the manual fixed extracting ;;
;aperature diameter ;; ymanfix2 = 1 = fix aperture to ymanwidth

   ;; The program will read in the following parameters from your
   ;; input table:  [orders do not matter, but do not change the
   ;; variable names]
;;;   slitno = slit number   format = i
;;;  objn   =  object name   format = a
;;;  zp   =  photometric redshift  format = f
;;;  rax   =  RA   format = a
;;;  decx   =  DEC   format = a
;;;   ymanguess = initial guess for the  emission line center  y-position on the slitlet,format=f
;;;   ymanwidth =initial guess for the  emission line width (radius) in pixels,format=f
;;;   lineguess = the initial guess for the observed wavelength of
;;;   the    strong lines:   6563,5007,4861,3727
;;;    flag=   : 6563=if you think it's Ha; 5007 if OIII, 3727 if OIIm 4861 if Hbeta
;;         0=star or don't fit the line; 999= an unknown
;;         emission line; -999 =no obvious detection but still want to
;;         fit a Gaussian at the location of the initial guess, ,format=f 
;;;   idm = multiple object id on the slit, if only one object on the slit, then =1, if multiple
;;              object, then id them as 2, 3, 4... format= i
;;;   ymanfix =  if you want to  fix the center of the 1D spec
;;;   extraction aperture,  1=fix, -1= do not fix ,format=i
;;;   ymanfix2 =  if you want to fix  the  width of the 1D spec
;;;   extraction aperture, 1=fix -1 = do not fix ,format=i
 ;;;;  PS your initial input table can contain as many columns as you
 ;;;;  want, as long as it contains the above 12 columns.  

ffin =               '(a,    a   ,      i,     a,  a,    a,  d,  f,      f,       f,           f,      i,    i,   d,    i, i,     i,       i)'
readcol,pathdat+intab, rundum,maskdum, slitno,objn,rax,decx,zp, kmag, ymanguess, ymanwidth, lineguess, flag,idm, zspec,kmerr,qzz, ymanfix, ymanfix2,format=ffin
;                            a,    a   ,  i,     a,  a,    a,  f,  f,      f,       f,           f,      i,    i,   f,    i, i,     i,       i

                                ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; INPUT FORMAT END ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


endelse

ymanwidth_tmp=ymanguess-ymanguess ;; initialize the array
ymanfix2_tmp=ymanfix-ymanfix

if n_elements(ymanwidth) lt 1 OR n_elements(ymanfix2) lt 1 then begin   ;; If undefined like data before Jan. 2017
   ymanwidth=ymanwidth_tmp ;; = array of  0
   ymanfix2=ymanfix2_tmp   ;; = array of 0
endif
                                



path=datapath
maskname=outputlist

readcol,objlist_fc,fxx,format='a'   ;; object list ;; electron/s
readcol,objlist_std_fc,fxx_sdev,format='a'   ;;; standard deviation list ;; electron/s



filenofit=N_ELEMENTS(objn)   ;;; this allows multiple objects on the slit
fileno = n_elements(fxx)
;stop
      fits_read,fxx[0],scube0,header_obj,exten_no=0
      spec1d_d=total(scube0,2,/nan)
      wavelen_obj=getwlcd(spec1d_d,header_obj)   ;;; moircsfire 

if keyword_set(ps) then setps_a4  ;; [CHANGE]
if keyword_set(ps) then device,/col,/land,file=path+maskname+'_fit1dflux.eps'   ;;;; tmp test ;; [CHANGE]

if keyword_set(first) then  xguess=fltarr(filenofit)    ;; obj flag from input table e.g, 1, -999
if keyword_set(first) then  fileslit=path+maskname+'slitcen.dat'    ;; obj flag from input table e.g, 1, -999
if keyword_set(first) then  yfixornot_arr=fltarr(filenofit)    ;; obj flag from input table e.g, 1, -999


;filenofit=1
colflag=intarr(filenofit)    ;; obj flag from input table e.g, 1, -999
colflag_2=intarr(filenofit)    ;; obj flag from input table e.g, 1, -999
colzp=strarr(filenofit)    ;;; obj name e.g., SF_2050 from input table
colname1=strarr(filenofit)    ;;; obj name e.g., SF_2050 from input table
colmosname=strarr(filenofit)   ;; obj mask slit ID name
colapecen=fltarr(filenofit)    ;;; spatial center on 2D slitlets to extract the 1D spec
colapewid=fltarr(filenofit)    ;;; spatial width on 2D slitlets to extract the 1D spec
colflag_spatial=intarr(filenofit)  ;;; flag for spatial center/width to extract the 1D spec  ;;; good detection = 1, faint target = 0 I wont't trust =0
fitemission=fltarr(filenofit,100)     ;;; to store the best line fitting parameters
fitemissionerr=fltarr(filenofit,100)  ;;;  to store the error of the  best line fitting parameters
flimit=fltarr(filenofit,10)  ;;;  to store the error of the  best line fitting parameters

fitcount=0
FOR noi=0,fileno-1 DO BEGIN    ;; Loop  over each slitlet ID
; FOR noi=2,2 DO BEGIN
;print,' now fit SLIT (fake slit ID ) ',noi+1
     fits_read,fxx[noi],scube,hdr,exten_no=0
     fits_read,fxx_sdev[noi],scube_sdev,hdrv,exten_no=0
scube=scube/1.d-17
scube_sdev=scube_sdev/(1.d-17)  ;; scale flux by 1.d-17 ergs/s/cm2/Ang


ppos1=0
ppos2=strpos(fxx[noi],'.fits')
Namefxx=strmid(fxx[noi],ppos1,ppos2-ppos1)

ppos1=0
ppos2=strpos(fxx_sdev[noi],'.fits')
Namefxx_sdev=strmid(fxx_sdev[noi],ppos1,ppos2-ppos1)

;;dx=where(strmatch(fxx,'*SF_5829*',/fold_case),cc)
 
;textpos='eps_H_S'
stlen1=strlen(textpos)
pppos1=strpos(fxx[noi],textpos)
pppos2=strpos(fxx[noi],'_t_fc.fits')
slitnum_st=strmid(fxx[noi],pppos1+stlen1,2)   ;;; extract slit ID number from the filename

slitnum=STR2NUM(slitnum_st)*1
slitmatch=where(slitno eq slitnum,cnamet)


print,' now fit SLIT S_ ',slitnum_st


if cnamet gt 0 then begin
  if cnamet gt 1 then print,'    *****************multiobjects !!!*************************'
   FoR idmulti=0, cnamet-1 Do Begin  ;;; LOOP OVER MULITPLE OBJECTS ON THE SAME SLIT  ;; ENDFOR-2
;  flag2=flag[slitmatch[0]] & zp2=zp[slitmatch[0]] & lineguess2=lineguess[slitmatch[0]]
   flag2=flag[slitmatch[idmulti]] & zp2=zp[slitmatch[idmulti]] & lineguess2=lineguess[slitmatch[idmulti]]
   objnametmp=objn[slitmatch[idmulti]] & ymanguesstmp=ymanguess[slitmatch[idmulti]]  & ymanfixtmp=ymanfix[slitmatch[idmulti]]
   ymanfixtmp2=ymanfix2[slitmatch[idmulti]] & ymanfixwidth2=ymanwidth[slitmatch[idmulti]]
   
 multinameno=idmulti
 mtag=strc(multinameno+1)

;;                          J  11533-13517 => zs[1.30338-1.69962 for OIII]
;; MOSFIRE rough Kband range 19200 - 24040
;; MOSFIRE rough Hband range  14500 - 18220  
;; MOSFIRE rough Jband range 11530 - 13520
;; MOSFIRE rough Yband range 9750 - 11240
;;;;;;;; THE PURPOSE OF THE FOLLOWING LINES IS TO PROVIDE A GOOD
;;;;;;;; INITIAL GUESS FOR THE REDSHIFT - HOWEVER, THESE LINES ARE NOT VERY
;;;;;;;;                                  NECESSARY IF YOU PROVIDE
;;;;;;;;                                  INITIAL TABLES; INTITIAL
;;;;;;;;                                  GUESS FROM YOUR INPUT
;;;;;;;;                                  TABLE OVERRIDES THE
;;;;;;;;                                  FOLLOWING: 
 
;; if [OIII] fall in Hband, then z = [1.91,2.63], if Ha fall in Hband
;; then z =[1.22, 1.77]  ;
;; if [OIII] fall in Kband, then z = [2.79,3.79], if Ha fall in Kband
;; then z =[1.895, 2.657]  ;
     flag2_2=flag2
;; initialize guessing
    if flag2 eq 6563 or flag2 eq 999 or flag2 eq 0 then zs=(lineguess2-6564.61)/6564.61 
    if flag2 eq 5007 then zs=(lineguess2-5008.24)/5008.24
    if flag2 eq -999 then zs=(lineguess2-6564.61)/6564.61
    if flag2 eq 4861 then zs=(lineguess2-4862.69)/4862.69

;;; fine tunning guess
   IF defined(bandK) then begin
      If flag2 eq 6563  then  zs=(lineguess2-6564.61)/6564.61
       If flag2 eq 3727  then  zs=(lineguess2-3727.09)/3727.09 

      If flag2 eq 999 then begin
           if zp2 lt 2.657 then zs=(lineguess2-6564.61)/6564.61;zp2
           if zp2 ge 2.79 AND zp2 le 3.79 then begin
            zs=(lineguess2-5008.24)/5008.24
            flag2_2=5007
           endif
      Endif
      If flag2 eq -999 then begin
           if zp2 lt 2.657 then zs=(lineguess2-6564.61)/6564.61
           if zp2 ge 2.79 AND zp2 le 3.79 then begin
            zs=zp2
            flag2_2=5007
           endif
      Endif
      If flag2 eq 0 AND zp2 le 2.5 then zs=2.2*1.d0
      If flag2 eq 0 AND zp2 gt 2.5 then begin
         zs=(lineguess2-5008.24)/5008.24
                     flag2_2=5007
      Endif
    Endif



;; fine tunning guess
   IF defined(bandJ) then begin
      if flag2 eq 4861 then   begin
       zs=(lineguess2-4862.69)/4862.69
                    flag2_2=5007

      endif

      If flag2 eq 3727 then begin
       zs=(lineguess2-3727.09)/3727.09 
       flag2_2=3727
     EndIf


      If flag2 eq 6563  then  zs=(lineguess2-6564.61)/6564.61
      If flag2 eq 5007 or flag2 eq 0  then begin
              zs=(lineguess2-5008.24)/5008.24
                    flag2_2=5007
      Endif 
      If flag2 eq 999 then begin
           if zp2 lt 2.657 then zs=(lineguess2-6564.61)/6564.61;zp2
           if zp2 ge 2.79 AND zp2 le 3.79 then begin
            zs=(lineguess2-5008.24)/5008.24
            flag2_2=5007
           endif
      Endif
      If flag2 eq -999 then begin
           if zp2 lt 1.0 then zs=(lineguess2-6564.61)/6564.61
           if zp2 ge 1.303 AND zp2 le 1.70 then begin
         ;   zs=zp2
              zs=(lineguess2-5008.24)/5008.24
            flag2_2=5007
           endif
           if zp2 gt 4 then begin   ;fake it
              zs=(lineguess2-3727.09)/3727.09 
                         flag2_2=3727
           endif
 
         if zp2 ge 1.7 and zp2 le 2.7 then begin
                zs=(lineguess2-4862.69)/4862.69
            flag2_2=5007
            endif
 
      Endif
    Endif





   IF Defined(bandH) then begin

      If flag2 eq 6563  then begin
       zs=(lineguess2-6564.61)/6564.61 
       flag2_2=6563
      Endif
     If flag2 eq 5007 or flag2 eq 999 then begin
      zs=(lineguess2-5008.24)/5008.24
      flag2_2=5007
     Endif

     If flag2 eq 4861 then begin
          zs=(lineguess2-4862.69)/4862.69
           flag2_2=5007
      Endif     
    
   If flag2 eq 4959 then begin
        zs=(lineguess2-4960.30)/4960.30
           flag2_2=5007
        endif
 
      If flag2 eq -999  then begin
          if zp2 lt 2.63 then begin
           zs=(lineguess2-5008.24)/5008.24;zp2
           flag2_2=5007
          endif  
         if zp2 ge 2.95 then zs=2.2
     Endif
     If flag2 eq 0 then begin
     zs=2.2
     flag2_2=5007
     endif
     If flag2 eq 3727 then begin
       zs=(lineguess2-3727.)/3727.
       flag2_2=3727
     EndIf
   ENDIF

;;;;; wavelength coverage of Y = 9750 ang to 11238
;;;      so if OII then z = 1.61605 to 2.01529
;;;;          if OIII then z =0.9472 to 1.24446
;;;;;;          if Hb then 1.00506 to 1.31107
   IF Defined(bandY) then begin
      If flag2 eq 3727  then  begin
           zs=(lineguess2-3727.09)/3727.09 
                     flag2_2=3727
      Endif
      If flag2 eq 4861 then begin
          zs=(lineguess2-4862.69)/4862.69
           flag2_2=5007
      Endif

      If flag2 eq 999 then begin
           if zp2 ge 1.61605 AND zp2 le 2.01529 then zs=(lineguess2-3727.09)/3727.09 
           if zp2 le 1.31107 AND zp2 ge 0.9472 then begin
            zs=(lineguess2-5008.24)/5008.24
            flag2_2=5007
           endif
      Endif

      If flag2 eq -999 then begin
  
          if zp2 ge 2.01529 then zs=1.8 ;; to avoid edge
           if zp2 lt 2.015 and zp2 gt 1.61 then begin
            zs=zp2
            flag2_2=3727
         endif

             if zp2 le 1.617 and zp2 ge 1.311 then begin
               zs=1.61
               flag2_2=3727
              endif
           if zp2 le 1.31107 AND zp2 ge 0.9472 then begin
            zs=(lineguess2-5008.24)/5008.24
            flag2_2=5007
           endif
      Endif
      If flag2 eq 0 then zs=1.69    
    Endif






   colname1[fitcount]=objn[slitmatch[idmulti]]
   colflag[fitcount]=flag2
   colflag_2[fitcount]=flag2_2
   colmosname[fitcount]=Namefxx
   colzp[fitcount]=zp2



FWHM_gal=5.  ;;; FOR PLOTTING RANGE PURPOSES SO DO NOT NEED TO BE ACCUARTE, KEEP IT AT 2.5-8 IS GOOD; MOSFIRE instrument FWHM measured by averaging skylines
   ;;;; PLEASE FINE-TUNE THE FOLLOWING INITIAL GUESS FOR CURVEFIT IF
   ;;;; THE FITTING IS NOT GOOD
; pro HaNIIdoublet, x, par, f, pder
; a=[z, fha,sigma, fn2,cont]    ;; alast
;     0    1     2         3    4

; pro double_OII, x, par, f, pder
; a=[z,fo21,fo22,sigma,cont] ;;;;  alastOII_d
;      0  1     2      3         4

; pro singlet_O3, x, par, f, pder
; a=[z,fha,sigma,cont] ;;; not used 
;      0   1    2         3

; pro double_O3, x, par, f, pder
; a=[z,f5007,sigma,cont]    ;;  alastR23
;     0   1        2         3
    
; pro O3Hb, x, par, f, pder 
; a=[z,f5007,sigma,fhb,cont]   ;;;; alastR23
;      0     1      2         3   4

; pro double_SII, x, par, f, pder  
; a=[z,f6717,f6731, sigma, cont];;;;  alastSII
;     0    1       2          3,         4

alast = [zs, 0.5d, inswid/2., 0.1d,0.003]    ; velinredshift, fluxHa, sigma, fluxNII,continuum
alastSII = [zs,0.4d, 0.1d,inswid/2.,0.003]  ;par =[z,f6717,f6731,sigma,cont]
alastR23 = [zs,2.0d,inswid/2.,0.01,0.003]   ;par=[z,FO3_5007,sigma,Fhb,cont]       if Hbeta and OIII are fitted together  
alastR23_2 = [zs,2.0d,inswid/2.,0.001]   ;par =[z,FO3_5007,sigma,cont]
alastOII_d = [zs, 0.3d,0.3d,inswid/2.,0.003]   ;par =[z,FOII_3727,FOII_3729,sigma,cont]

parnOIId=n_elements(alastOII_d)
parnHaNII=n_elements(alast)
parnSII=n_elements(alastSII)
parnR23=n_elements(alastR23)
parnR23_2=n_elements(alastR23_2)


;;;; ;;; FOR PLOTTING RANGE PURPOSES
Ewlow = 6360.d0*(1.+zs) & Ewhi = 6900.d0*(1.+zs)      ;; [CHANGE]
EwlowS = (6716-80)*(1.+zs) & EwhiS = (6731+120)*(1+zs) ;; [CHANGE]
EwlowR23 = 4650.*(1.+zs) & EwhiR23 = 5100.*(1.+zs)
EwlowOII =3710*(1.+zs)  & EwhiOII = 3750.*(1+zs)     ;; [CHANGE]

 
  spec2dcut=scube ;[where(wavex ge lowcut and wavex le hicut),*]
 ; clean=where(spec2dcut gt 11*median(spec2dcut[220:1800,*]),ctx)
  ;if ctx gt 1 then spec2dcut[clean]=median(spec2dcut[220:1800,*])
 
;if noi eq 0 then
;space_profile=median(spec2dcut[837:1100,*],dimension=1)
;cutw1=19700. & cutw2=22900. 

;; ;;;; OLD METHOD OF DOING THE SPATIAL CUT
;; if defined(bandJ) then begin
;; if noi ge 0 then  space_profile=median(spec2dcut[50:1400,*],dimension=1)
;; endif

;; if defined(bandH) then begin
;; if noi ge 0 then  space_profile=median(spec2dcut[300:1600,*],dimension=1)
;; endif


;; if (~defined(bandJ) AND ~defined(bandH)) OR defined(bandK) then begin
;; if noi ge 0 then  space_profile=median(spec2dcut[250:1700,*],dimension=1)
;; endif
;; ;;;; OLD METHOD OF DOING THE SPATIAL CUT

cutrange=50   ;; do a cut of 300*2 ang. around the guessed emission line position
cut_r=where((wavelen_obj gt lineguess2-cutrange>min(wavelen_obj)) AND (wavelen_obj lt lineguess2+cutrange<max(wavelen_obj)),countt)
if countt lt 1 then begin
print,'   cannot find proper spatial range stop'
stop
endif
cutlowwl=cut_r[0] & cuthighwl=cut_r[countt-1]

if noi ge 0 then  space_profile=median(spec2dcut[cutlowwl:cuthighwl,*],dimension=1)



  judge_spatial=1    ;;; good detection = 1, faint target = 0
  colflag_spatial[fitcount]=judge_spatial


ysize_spatial=n_elements(space_profile)

fitcen=1  ;;;;; you can set this to be > 10 to let the program find the line center automatically....
;;;;
IF keyword_set(ps) then begin  ;;; FIND THE SPATIAL PEAK AUTOMATICALLY 
  if keyword_set(first) then fitcen=-999  ;; obj flag from input table e.g, 1, -999

  if fitcen ge 10 then begin 
    x=where(space_profile eq max(space_profile),ctt)
    y=1
;   if ctt gt 1 AND ctt lt 3 then x=x[0]    ;;; maximum of 2 peaks allowed
;   if (ctt eq 1) then begin
       if ctt eq 1 AND (x[0] le 10 or x[0] gt (ysize_spatial-10)) then begin
          x=ysize_spatial/2  & cen=x & width=2.   ;;; if not fitting spatial profile, then use middle row and stellar profile
         judge_spatial=0
         colflag_spatial[fitcount]=judge_spatial
       endif
;   endif

 
   If ctt le 0 OR ctt ge 4  then begin   ;; for weird spatial profiles or target on the edge
     x=x[0]
     x=ysize_spatial/2  & cen=x & width=2.   ;;; if not fitting spatial profile, then use middle row and stellar profile
     judge_spatial=0
     colflag_spatial[fitcount]=judge_spatial
    ; GOTO, Jump1 
  Endif
  endif

Endif else begin   ;;; OR DO IT BY HAND


if keyword_set(first) then begin
   if keyword_set(interact) then begin

  !p.multi=0

;  plot,space_profile,title='Spatial Profile of slitID '+slitnum_st+'-'+mtag,charsize=2,xtitle=' Y (pixel)'
    plot,space_profile,title='Spatial Profile of object '+objnametmp+'-'+mtag,charsize=2,xtitle=' Y (pixel)'

   print,'Click at a peak of the profile or where the object should be...'
  print,' if one slit has multiple objects, the same plot will show up multiple times, click on the spatial position in the same order as your input table file mask*_data*.dat'
  cursor,x,y
  wait,0.1
  xguess[fitcount]=x
    endif else begin
  xguess[fitcount]=ymanguesstmp
yfixornot_arr[fitcount]=ymanfixtmp
endelse 

  GOTO, Jump1 
  endif

  plot,space_profile,title='Spatial Profile of obj spec.'
  print,'Click at a peak of the profile or where the object should be...'
  cursor,x,y
  wait,0.1

Endelse
 
readcol,path+maskname+'slitcen.dat',xguessarr,yfixornot_arr_r,skipline=1,format='f'   ;; obj flag from input table e.g, 1, -999
;readcol,path+maskname+'slitcennote.dat',xguessarr,skipline=1,format='f'   ;; obj flag from input table e.g, 1, -999

checksize=size(xguessarr) & checknox=checksize[1]

if checknox ne filenofit then print,' spatial guess file does not match number of objects, go back and check ---' 
if checknox ne filenofit then stop
 
x=xguessarr[fitcount] & xcenfix=x
yfixornot=yfixornot_arr_r[fitcount]
;stop
  space_profile_cut=space_profile[x-20 > 0:x+20 < (ysize_spatial-1)]
  xarray=findgen(41)-20.
   xladds=abs(x-21) & yladds=x+20-ysize_spatial+1
   if x lt 20 then xarray=xarray[xladds:40]
     if x gt (ysize_spatial-1)-20 then xarray=xarray[0:40-yladds]
;; check nan values
     nanindexc=where(finite(space_profile_cut,/nan),nanindexcc)
     if nanindexcc gt 0 then space_profile_cut[nanindexc]=0
loadcolors

judge_spatial=1
  ;if colname1[fitcount] eq '24um_838015b' then stop
  ; if judge_spatial eq 1 then begin
 yfit=mpfitpeak(xarray,space_profile_cut,A,sigma=sigma,nterms=4,/positive,errmsg=errmsg)
   plot,xarray,space_profile_cut,psym=10,title=' Spatial Profile Used for Extracting 1D spec',thick=4,xtitle='Y-pixel';,xtickformat='(A1)'
   oplot,xarray,yfit,color=2
   xyouts,-11,max(space_profile_cut)*0.999,'objname= '+colname1[fitcount]
   x+=a[1]
   cen=x
   width=a[2]*1.1774d0   ;;; turn this into a FWHM

   ;;  inswid
    ;;;;;;CHANGE THIS TO SET THE MINIMAL AND MAX SPATIAL PROFILE FOR
    ;;;;;;1D EXTRACTION
   ;;;; you can also use yfixornot=1 in your input table to enforcing
   ;;;; a manual number for extracting aperture 
   seeing=0.8 & scale_ph=0.18 ;;; 
   FWHMmin= seeing/scale_ph ;; in pixel unit ;; using seeing size in pixels as the minimal extracting aperture 2.55*2
    FWHMmax=ysize_spatial/2  ; 10.*2     ;;; I set it to half the length of the slit meanining assuming your object is no larger than half the size of the slit-length, if it is larger than this, the dithering won't work anyways;; ;Alternatively, you can set this to your dithering length
   ;;                ;;; ;Rejecting criteria for spatial fitting ---
   ;;                if profile TOO narrow or TOO wide                          
  okay=where(finite(width,/nan) OR finite(width,/infinity) OR width lt FWHMmin/2. OR width gt FWHMmax/2. OR cen ge (ysize_spatial-6) OR cen le 3,cx)
 ;;;;;;             not a good fit  OR  not a good fit     Or too
 ;;;;;;                                                     narrow (less than seeing)
  ;;;;;                                                                           OR too wide(larger than half slit length=> dithering failed)
  ;;;;                                                                                                  OR    center is too close to the edges
;;   bad fit  OR manual fix
 ; ymanfixtmp2
  if (cx ge 1 or yfixornot gt 0) AND (ymanfixtmp2 le 0)  then begin
       width = 2.55 & cen=xcenfix & colflag_spatial[fitcount]=0 
       print,'No good spatial fit -->  use fixed center and stellar FWHM', cen, width*2+1         
    endif  

   if (cx ge 1 or yfixornot gt 0) AND (ymanfixtmp2 gt 0)  then begin
       width = ymanfixwidth2 & cen=xcenfix & colflag_spatial[fitcount]=0
       print,'No good spatial fit -->  use fixed center and user defined aperture diameter', cen, width*2+1       
    endif  


  
   
   if (cx lt 1) AND (ymanfixtmp2 le 0) then begin
    width = width & cen=cen & colflag_spatial[fitcount]=1 
     print,' use center and extracting aperture diameter from fitting',cen,width*2+1
  endif


    if (cx lt 1) AND (ymanfixtmp2 gt 0) then begin
    width = ymanfixwidth2 & cen=cen & colflag_spatial[fitcount]=1 
     print,' use center from fitting and user defined aperture diameter',cen,width*2+1
   endif

      if abs(cen-xcenfix) gt 8  then begin ;;; NOTE: width= 2-2.75 => seeing 0.7-1", i.e., using the seeing FWHM size as the minimal aperture here
       width = FWHMmin/2. & cen=xcenfix & colflag_spatial[fitcount]=0 
       print,'Fitted center is 8pixel away so not fitting -->  use fixed center and stellar FWHM ( = 5 pixels for this observation.... )'
      endif

      print, 'check --------------------', ymanfixtmp2, ymanfixwidth2

      if  ymanfixtmp2 gt 0 then begin
         print, slitnum_st
         width = ymanfixwidth2
          endif

      
    
colapecen[fitcount]=cen & colapewid[fitcount]=width*2+1
xyouts,-20,max(space_profile_cut)*0.5, 'center, FWHM (pixel)= '

if colflag_spatial[fitcount] eq 0 then begin
xyouts,-20,max(space_profile_cut)*0.3, 'no fit, used stellar profile'
endif

xyouts,-20,max(space_profile_cut)*0.4,'     '+strmid(strc(cen),0,3)+' , '+strmid(strc(width*2+1),0,2)
xyouts,-20,min(space_profile_cut)*0.5,'eye flag='+strmid(strc(flag2),0,6),color=5,charthick=4
 




;;;;;;;;;;;;; FROM 2D to 1D  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; if slitnum_st eq 15 then width=2  ; twick for maskH3-2016

;; check for problematic slits; change NAN to 0 so that the
;; code won't crash ...
      scheck=size(spec2dcut)
        xscheck=scheck[1] & yscheck=scheck[2]
        nanindexc2=where(finite(spec2dcut,/nan),nanindexcc2)
        levbad=xscheck*yscheck-10
     if nanindexcc2 gt levbad then begin
         spec2dcut=fltarr(xscheck,yscheck)
        scube_variance=fltarr(xscheck,yscheck)
        print,' this is a bad slit     ', Namefxx
     endif   



     ;print,' now fit SLIT S_ ',slitnum_st

  spec2d_cut=spec2dcut[*,cen-width>0:cen+width< (ysize_spatial-1)]
  spec1d=total(spec2d_cut,2,/nan);/1.d-17
  scube_variance=scube_sdev^2
  var2d_cut=scube_variance[*,cen-width>0:cen+width< (ysize_spatial-1)]
  var1d=sqrt(total(var2d_cut,2,/nan));;;/(1.*width)
  wavex=getwlcd(spec1d,hdr)   ;;; mosfire
  varf=var1d   ;;1 sigma spectrum!
 

;  if slitnum_st eq '06' then begin
;     help,spec1d
;     set_plot,'x'
;     plot,wavex,spec1d
;stop
;endif
     
sz1=size(wavex)




if defined(bandJ) then begin
rejmed=median(spec1d[where(wavex gt 12000. and wavex le 12550.)]) 
rejvar=moment(spec1d[where(wavex gt 12000. and wavex le 12550.)]) 

endif



if defined(bandH) then begin

rejmed=median(spec1d[where(wavex gt 16100. and wavex le 16600.)]) 
rejvar=moment(spec1d[where(wavex gt 16100. and wavex le 16600.)]) 

endif


if (~defined(bandJ) AND ~defined(bandH) AND ~defined(bandY)) OR defined(bandK) then begin
rejmed=median(spec1d[where(wavex gt 20000. and wavex le 23000.)]) 
rejvar=moment(spec1d[where(wavex gt 20000. and wavex le 23000.)]) 

endif


if defined(bandY) then begin

rejmed=median(spec1d[where(wavex gt 9989. and wavex le 10945.)]) 
rejvar=moment(spec1d[where(wavex gt 9989. and wavex le 10945.)]) 

endif

rejdex=where(abs((spec1d-rejmed)) gt 5*sqrt(rejvar[1]),ct)
;rejdex=where(abs(spec1d) gt 60*rej,ct)   ;;; reject abnormal pixels    ;;; ***### This step could be done smarter, refine later!!!!

;;; Do not do any rejection ;;;;;;
;;   if ct gt 1 then spec1d[rejdex] = spec1d[rejdex]/max(abs(spec1d[rejdex]))


;;;;; write 1D spectrum into a file, put stdeviation into extension
;;        mwrfits,scube,path+Namefxx+'_t.fits',hdr,/create

sxaddpar,hdr,"fluxUnit",' 10^(-17)ergs/s/A/cm^2'
cdel1 = sxpar(hdr,"CDELT1")     ;; DEC tan xori
cd1_1 = sxpar(hdr,"CD1_1")     ;; DEC tan xori
crpix1=sxpar(hdr,"CRPIX1")
crval1=sxpar(hdr,'CRVAL1')

hdr1d=hdr
mkhdr,hdr1d,spec1d
sxaddpar,hdr1d,"fluxUnit",' 10^(-17) ergs/s/A/cm^2'
sxaddpar,hdr1d,"ext_numbers",'Three'
sxaddpar,hdr1d,"ext-0",'flux'
sxaddpar,hdr1d,"ext-1",'error'
sxaddpar,hdr1d,"ext-2",'wavelength in Angs'
sxaddpar,hdr1d,"CUNIT1",'Angs'
sxaddpar,hdr1d,"CRVAL1",crval1
sxaddpar,hdr1d,"CRPIX1",crpix1
sxaddpar,hdr1d,"CDELT1",cdel1
sxaddpar,hdr1d,"CD1_1",cd1_1


if multinameno eq 0 then begin

sxaddpar,hdr1d,"EXTEND",'T'
  mwrfits,spec1d,Namefxx+'_1D.fits',hdr1d,/create
hdr1d_1=hdr1d
sxaddpar,hdr1d_1,"stdeviation",' funit2'
  mwrfits,varf,Namefxx+'_1D.fits',hdr1d_1
hdr1d_2=hdr1d 
 sxaddpar,hdr1d_2,"wavelength",' Ang'
  mwrfits,wl,Namefxx+'_1D.fits',hdr1d_2
endif else begin
hdr1d=hdr1d
sxaddpar,hdr1d,"EXTEND",'T'
  mwrfits,spec1d,Namefxx+'_1D-'+mtag+'.fits',hdr1d,/create
hdr1d_1=hdr1d
sxaddpar,hdr1d_1,"stdeviation",' funit2'
  mwrfits,varf,Namefxx+'_1D-'+mtag+'.fits',hdr1d_1
hdr1d_2=hdr1d 
sxaddpar,hdr1d_2,"wavelength",' Ang'
  mwrfits,wl,Namefxx+'_1D-'+mtag+'.fits',hdr1d_2
endelse


;;;;;;;;;;;

sm1d=smooth(spec1d,1)   ; smooth the spectra by 3 spectral pixel = 4.89 Ang before fitting, adjust yourself
plot,wavex,spec1d,charsize=1.,title=maskname+'-'+slitnum_st+'-'+mtag,xra=[min(wavex)+10,max(wavex)-10],yra=[min(spec1d),max(spec1d)],/xs,/ys,/nodata,thick=4,charthick=4 $
           ,xtitle='Wavelength (Ang)',ytitle=' 10^-17 ergs/s/cm2/Ang'
 wl=wavex 

        wl = CONGRID(wl,sz1[1]/1)
        sm1d = CONGRID(sm1d,sz1[1]/1)
        varf = CONGRID((var1d),sz1[1]/1)
        oplot,wl,sm1d,psym=10,thick=2
        oplot,wl,varf,psym=3,thick=0.6,color=4
xyouts,2.0e4,max(spec1d)*0.9,'  green = RMS; black =  data'
loadct,39


;;; ttttttttttttcheck   point 1111111
If flag2_2 ne 5007 AND flag2_2 ne 3727 then begin
        ;;;;; First NII Ha complex
;  PARINFO Example:
    parinfo = replicate({value:0.D, fixed:0, limited:[0,0],limits:[0.D,0]}, n_elements(alast))
    parinfo(0).limited(0) = 1
    parinfo(0).limits(0)  = zs-0.1D
    parinfo(0).limited(1) = 1
    parinfo(0).limits(1)  = zs+0.1D
    parinfo(1).limited(0) = 1
    parinfo(1).limits(0)  = 1.D-8  ;;no negative Halpha flux 
    parinfo(2).limited(0) = 1
    parinfo(2).limits(0)  = 1.D-8  ;; dispersion lower cut
    parinfo(2).limited(1) = 1            
    parinfo(2).limits(1)  = 30.d0  ;; dispersion upper cut 
    parinfo(3).limited(0) = 1
    parinfo(3).limits(0)  = 1.D-8  ;; no negative NII flux 
    parinfo(*).value = alast  ; [zs, 0.5d, 1, 0.1d,0.003]    ; velinredshift, fluxHa, sigma, fluxNII,continuum

     okha = where(wl ge Ewlow and wl le Ewhi,ct1)
     ;; if ct1 lt 1 then print,' fail to find proper range of the spectrum for fitting, redshift guess must be way off---- '
     If ct1 lt 1 AND defined(bandY) then begin
          plot,wl,sm1d,/xs,title='1D spec',charthick=2,charsize=1.2,thick=2,ytitle=' 10^-17 ergs/s/cm2/Ang';,symsize=.8                 
          Goto,jumpskiphas2
     Endif


      wltmp=wl[okha] & a1=alast
      ytmp=sm1d[okha] & tmpnoise=varf[okha]

      ;; If slitnum eq 15 then begin
      ;;    nky=n_elements(ytmp)
      ;;       ytmp = CONGRID(ytmp,nky/4)
      ;;       wltmp = CONGRID(wltmp,nky/4)
      ;;       tmpnoise = CONGRID(tmpnoise,nky/4)
      ;;               ytmp=smooth(ytmp,7) & wltmp=smooth(wltmp,7)
      ;;   ;    stop
      ;;    print,slitnum,'))))))))))'
      ;; endif

       
             tmpw= 1./(tmpnoise)^2 ; ;par =[z,Fha,sigma,Fnii,cont]
              nandull=where(finite(tmpw,/nan) OR finite(tmpw,/infinity),badnoisew)
       if badnoisew ge 1 then              tmpw[nandull]=1.e-6 ;; small weighting for NAN, INFINITY values
              fit1 = mpcurvefit(wltmp,ytmp,tmpw,a1,sigma1,chisq=chi1f1,dof=dof1,function_name="HaNIIdoublet",/noderivative,/quiet,parinfo=parinfo)           ; 
;;;;              ;; correct for instrument width
              a1[2] = sqrt(a1[2]^2-(inswid/2.3548)^2)
              sigma1[2] = sigma1[2]/sqrt(2.d*a1[2])
if multinameno eq 0 then begin
  mwrfits,wltmp,Namefxx+'_1D_HaNIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D_HaNIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D_HaNIIbestfit.fits',hdr1d,/create
endif else begin
  mwrfits,wltmp,Namefxx+'_1D-'+mtag+'_HaNIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D-'+mtag+'_HaNIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D-'+mtag+'_HaNIIbestfit.fits',hdr1d,/create
endelse
;;;;
tagx=' nofit' & tagok=' fit'
if flag2 eq 0 then   tagp=tagx
if flag2 ne 0 then   tagp=tagok

     plot,wltmp,ytmp,xra=[Ewlow,Ewhi],/xs,title='Ha-[NII] line'+tagp,charthick=2,charsize=1.2,thick=2,ytitle=' 10^-17 ergs/s/cm2/Ang';,symsize=.8                 
                oplot,wltmp,tmpnoise,color=150,thick=1,psym=3

     if flag2 ne 0 then begin
        oplot,wltmp,fit1,color=50,thick=4
              xyouts,Ewlow+FWHM_gal,min(ytmp)*0.8,'S/N (Ha,NII)='+strmid(strc((a1[1]/sigma1[1])),0,4)+','+strmid(strc((a1[3]/sigma1[3])),0,4),charthick=4,color=6
     xyouts,Ewlow+FWHM_gal,max(ytmp)*0.999, ' zspec_Ha= '+strmid(strc(a1[0]),0,5),charthick=2
      endif


;;  if colname1[fitcount] eq 'SF_4647' then  begin
;; stop
;; endif
     fitemission[fitcount,0:parnHaNII-1]=a1
     fitemissionerr[fitcount,0:parnHaNII-1]=sigma1

;;  if colname1[fitcount] eq 'SF_4647' then  begin
;; stop
;; endif

     zs=a1[0]
     oplot,[6564.61*(1+zs),6564.61*(1+zs)],[-100,100],linestyle=2     
     oplot,[6585.23*(1+zs),6585.23*(1+zs)],[-100,100],linestyle=2     
     oplot,[6549.84*(1+zs),6549.84*(1+zs)],[-100,100],linestyle=2     


     noisecen=6564.61*(1+zs) & noisewid=6.8   ;; 6 ang. mimal sigma width estimated from fitting strong lines, about 100km/s at z=2
     oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
     noisewid2=a1[2]  
     if noisewid2 lt 2  OR  noisewid2 gt 30 then noisewid2=6.8
     if ctn ge 1 then flimit[fitcount,0]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)  ;;fha limit
     noisecen=6585.23*(1+zs) & noisewid=6.8
     oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
     if ctn gt 1 then begin
       flimit[fitcount,1]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fnii limit
     endif else begin
      flimit[fitcount,1]=0  ;; fnii limit
      ;flimit[fitcount,1]=median(sm1d[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fnii limit
     endelse

;stop
;; if colname1[fitcount] eq 'SF_BKG_4123' then  begin
;; print,' ++++++++++++++++++++++++++'
;; print, colname1[fitcount], ctn
;;        flimit[fitcount,1]=median(ytmp[oknoise])*noisewid*sqrt(2.*!pi)   ;; fnii limit

;; stop
;; endif


        

;Endelse ;;;;????????????

;  PARINFO Example:
    parinfo = replicate({value:0.D, fixed:0, limited:[0,0],limits:[0.D,0]}, n_elements(alastSII))
    parinfo(0).limited(0) = 1
    parinfo(0).limits(0)  = zs-0.02D
    parinfo(0).limited(1) = 1
    parinfo(0).limits(1)  = zs+0.02D
    parinfo(1).limited(0) = 1
    parinfo(1).limits(0)  = 1.D-8  ;; flux lower cut 2.d-10= 0 
    parinfo(2).limited(0) = 1
    parinfo(2).limits(0)  = 1.D-8  ;; dispersion lower cut 2.d-10= 0 km/s
    parinfo(3).limited(0) = 1
    parinfo(3).limits(0)  = 1.d-8 ;; flux lower limit cut 2.d-10= 0
    parinfo(3).limited(1) = 1
    parinfo(3).limits(1)  = 30.d0 ;; flux lower limit cut 2.d-10= 0
 
    parinfo(*).value = alastSII  ;alastSII = [zs,0.4d, 1d,4.,0.003]  ;par =[z,f6717,f6731,sigma,cont]
 
       ;;;;; SII complex
    wl=wavex 
    okha = where(wl ge EwlowS and wl le EwhiS,ct1)
 
    if ct1 lt 1 or ctn lt 1 then begin
        print,' warning -- fail to find proper range of spectra to fit  SII, do a fake plot' 
               plot,wl,sm1d,xra=[EwlowS,EwhiS],/xs,title='[SII] emission'+tagp,charthick=2,charsize=1.2,thick=2,ytitle=' 10^-17 ergs/s/cm2/Ang';,    
                  oplot,[6718.32*(1+zs),6718.32*(1+zs)],[-100,100],linestyle=2     
                  oplot,[6732.17*(1+zs),6732.17*(1+zs)],[-100,100],linestyle=2     

;;no fitting 0 array dummy
          fitemission[fitcount,parnHaNII+0:parnSII+parnHaNII-1]=a1-a1
          fitemissionerr[fitcount,parnHaNII+0:parnSII+parnHaNII-1]=sigma1-sigma1
          flimit[fitcount,2]=0;median(tmpnoise[oknoise])*noisewid*sqrt(2.*!pi)   ;; fs6717 limit
          flimit[fitcount,3]=0;median(tmpnoise[oknoise])*noisewid*sqrt(2.*!pi)   ;; fs6717 limit




Endif else begin

           wltmp=wl[okha] & a2=alastSII
           ytmp=sm1d[okha] & tmpnoise=varf[okha]
              tmpw= 1./(tmpnoise)^2 ; ;par =[z,Fha,sigma,Fnii,cont]
              nandull=where(finite(tmpw,/nan) OR finite(tmpw,/infinity),badnoisew)
          if badnoisew ge 1 then              tmpw[nandull]=1.e-6 ;; small weighting for NAN, INFINITY values
          fit1 = mpcurvefit(wltmp,ytmp,tmpw,a2,sigma2,chisq=chi1f2,dof=dof2,function_name="double_SII",/noderivative,/quiet,parinfo=parinfo)   
; flimit[fitcount,1]=median(tmpnoise[oknoise])     
;;;;
          ;;;;              ;; correct for instrument width
              a2[3] = sqrt(a2[3]^2-(inswid/2.3548)^2)
              sigma2[3] = sigma2[3]/sqrt(2.d*a2[3])
if multinameno eq 0 then begin
  mwrfits,wltmp,Namefxx+'_1D_SIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D_SIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D_SIIbestfit.fits',hdr1d,/create
endif else begin
  mwrfits,wltmp,Namefxx+'_1D-'+mtag+'_SIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D-'+mtag+'_SIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D-'+mtag+'_SIIbestfit.fits',hdr1d,/create
endelse
;;;;

;if slitnum_st eq 7 then stop
checka1=N_elements(ytmp)
checka2=n_elements(fit1)
if checka1 eq checka2 then begin
               plot,wltmp,ytmp,xra=[EwlowS,EwhiS],/xs,title='[SII] emission fit',charthick=2,charsize=1.2,thick=2,ytitle=' 10^-17 ergs/s/cm2/Ang';,    
               oplot,wltmp,tmpnoise,color=150
               if flag2 ne 0 then begin
                               oplot,wltmp,fit1,color=50,thick=4
                               xyouts,EwlowS+FWHM_gal,min(ytmp)*0.89,'S/N='+strmid(strc((a2[1]/sigma2[1])),0,4)+','+strmid(strc((a2[2]/sigma2[2])),0,4),charthick=4,color=6
               endif
           oplot,[6718.32*(1+zs),6718.32*(1+zs)],[-100,100],linestyle=2     
             oplot,[6732.17*(1+zs),6732.17*(1+zs)],[-100,100],linestyle=2     
          noisecen=6718.32*(1+zs) & noisewid=6.8
           oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
     noisewid2=a2[3]  
        if noisewid2 lt 2 OR  noisewid2 gt 30 then noisewid2=6.8

          if ctn gt 1 then begin
          flimit[fitcount,2]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fs6717 limit
          endif else begin
          flimit[fitcount,2]=0
          endelse
          noisecen=6732.17*(1+zs) & noisewid=6.8
          oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
          if ctn gt 1 then begin
          flimit[fitcount,3]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fs6732 limit
          endif else begin
          flimit[fitcount,3]=0;median(tmpnoise[oknoise])*noisewid*sqrt(2.*!pi)   ;; fs6717 limit
          endelse
           fitemission[fitcount,parnHaNII+0:parnSII+parnHaNII-1]=a2
          fitemissionerr[fitcount,parnHaNII+0:parnSII+parnHaNII-1]=sigma2
 endif       
Endelse

Endif

jumpskiphas2:print,' -- fit o2 range'

;;;;######### add O2 emission begin

if flag2_2 eq 3727 then begin
;a3=alastR23
;sigma3=alastR23-alastR23+100.
       ;;;;;  HB OIII complex
;  PARINFO Example:
  parinfo = replicate({value:0.D, fixed:0, limited:[0,0],limits:[0.D,0]}, n_elements(alastOII_d))
  parinfo(0).limited(0) = 1
   parinfo(0).limits(0)  = zs-0.1D
   parinfo(0).limited(1) = 1
   parinfo(0).limits(1)  = zs+0.1D
   parinfo(1).limited(0) = 1
   parinfo(1).limits(0)  = 1.D-8  ;; flux lower cut  
   parinfo(2).limited(0) = 1
   parinfo(2).limits(0)  = 1.D-8  ;; flux lower cut 
   parinfo(3).limited(0) = 1            
   parinfo(3).limits(0)  = 1.D-8  ;; dispersion lower limit 
   parinfo(3).limited(1) = 1
   parinfo(3).limits(1)  = 40.d0  ;;  dispersion lower limit 
   parinfo(*).value = alastOII_d  ;;par =[z,FOII_3727,FOII_3729,sigma,cont]
 

 okha = where(wl ge EwlowOII and wl le EwhiOII,ct1)
 noisecen=(EwlowOII+EwhiOII)/2 & noisewid=30
 oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)

if ct1 lt 1 or ctn lt 1 then print,' -- fail to find proper range of spectra to fit OII- region' 
if ct1 lt 1 or ctn lt 1 then stop



 wltmp=wl[okha] & a3_o2=alastOII_d & sigma3_o2=a3_o2-a3_o2+1.e-5

 ytmp=sm1d[okha] & tmpnoise=varf[okha]
 
                       tmpw= 1./(tmpnoise)^2 ; ;par =[z,Fha,sigma,Fnii,cont]
              nandull=where(finite(tmpw,/nan) OR finite(tmpw,/infinity),badnoisew)
       if badnoisew ge 1 then              tmpw[nandull]=1.e-6 ;; small weighting for NAN, INFINITY values

            fit1 = mpcurvefit(wltmp,ytmp,tmpw,a3_o2,sigma3_o2,chisq=chi1f2,dof=dof2,function_name="double_OII",/noderivative,/quiet,parinfo=parinfo)  
         ;      flimit[fitcount,2]=median(tmpnoise[oknoise])
                                ;
            ;;;;              ;; correct for instrument width
              a3_o2[3] = sqrt(a3_o2[3]^2-(inswid/2.3548)^2)
              sigma3_o2[3] = sigma3_o2[3]/sqrt(2.d*a3_o2[3])
              
                plot,wltmp,ytmp,xra=[EwlowOII,EwhiOII],/xs,title='OII lines zoom-in'+tagp,charthick=2,charsize=1.2,thick=4,ytitle=' 10^-17 ergs/s/cm2/Ang';,
                oplot,wltmp,tmpnoise,color=150
                if flag2 ne 0 then begin
                oplot,wltmp,fit1,color=50
                oplot,wltmp,ytmp-fit1,color=220
  xyouts,EwlowOII+FWHM_gal,max(fit1)*0.999, ' zspec_OII= '+strmid(strc(a3_o2[0]),0,5),charthick=2,color=5
             xyouts,EwlowOII+FWHM_gal,min(ytmp)*0.7,'S/N='+strmid(strc((a3_o2[3]/sigma3_o2[3])),0,4)+','+strmid(strc((a3_o2[1]/sigma3_o2[1])),0,4),charthick=4,color=6
                endif
             oplot,[3727.09*(1+zs),3727.09*(1+zs)],[-100,100],linestyle=2     
   oplot,[3729.88*(1+zs),3729.88*(1+zs)],[-100,100],linestyle=2    

                plot,wltmp,ytmp,xra=[EwlowOII,EwhiOII],/xs,title='OII lines zoom-in(replicate)',charthick=2,charsize=1.2,thick=4,ytitle=' 10^-17 ergs/s/cm2/Ang';,
                if flag2 ne 0 then begin
                   oplot,wltmp,fit1,color=50
                endif
;;;;
if multinameno eq 0 then begin
  mwrfits,wltmp,Namefxx+'_1D_OIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D_OIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D_OIIbestfit.fits',hdr1d,/create
endif else begin
  mwrfits,wltmp,Namefxx+'_1D-'+mtag+'_OIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D-'+mtag+'_OIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D-'+mtag+'_OIIbestfit.fits',hdr1d,/create
endelse
;;;;
     

 fitemission[fitcount,parnR23+parnSII+parnHaNII:parnOIId+parnR23+parnSII+parnHaNII-1]=a3_o2
 fitemissionerr[fitcount,parnR23+parnSII+parnHaNII:parnOIId+parnR23+parnSII+parnHaNII-1]=sigma3_o2
  oplot,[3727.09*(1+zs),3727.09*(1+zs)],[-100,100],linestyle=2     
   oplot,[3729.88*(1+zs),3729.88*(1+zs)],[-100,100],linestyle=2    

     noisecen=3727.09*(1+zs) & noisewid=6.8
     noisewid2=a3_o2[3]  
     if noisewid2 lt 2 OR  noisewid2 gt 30 then noisewid2=6.8

 oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
if ctn ge 1 then begin
flimit[fitcount,4]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fs6717 limit
endif else begin
flimit[fitcount,4]=0
endelse 

endif


;;;;########## add 02 emission end




if flag2_2 eq 5007 then begin
;a3=alastR23
;sigma3=alastR23-alastR23+100.
       ;;;;;  HB OIII complex
;  PARINFO Example:
  parinfo = replicate({value:0.D, fixed:0, limited:[0,0],limits:[0.D,0]}, n_elements(alastR23))
  parinfo(0).limited(0) = 1
   parinfo(0).limits(0)  = zs-0.1D
   parinfo(0).limited(1) = 1
   parinfo(0).limits(1)  = zs+0.1D
   parinfo(1).limited(0) = 1
   parinfo(1).limits(0)  = 1.D-8  ;; O3 flux lower cut 2.d-10= 0 
   parinfo(2).limited(0) = 1
   parinfo(2).limits(0)  = 1.d-8  ;; dispersion lower cut 2.d-10= 0 km/s
   parinfo(2).limited(1) = 1            
   parinfo(2).limits(1)  = 30.d0  ;; dispersion upper cut
   parinfo(3).limited(0) = 1
   parinfo(3).limits(0)  = 1.D-8  ;; Hb flux lower cut 2.d-10= 0 
   parinfo(*).value = alastR23  ; [zs, 0.5d, 1, 0.1d,0.003]    ; velinredshift, fluxHa, sigma, fluxNII,continuum
 

 okha = where(wl ge EwlowR23 and wl le EwhiR23,ct1)
     If ct1 lt 1 AND defined(bandY) then begin
          plot,wl,sm1d,/xs,title='1d spec',charthick=2,charsize=1.2,thick=2,ytitle=' 10^-17 ergs/s/cm2/Ang';,symsize=.8                 
          Goto,jumpskipo3
     Endif

;; If ct1 lt 1 AND defined(bandY) then begin

;; plot,wl,sm1d,/xs,title='1d spec',charthick=2,charsize=1.2,thick=2,ytitle=' 10^-17 ergs/s/cm2/Ang';,symsize=.8                 

;; Endif else begin
;; ;endif




 noisecen=(EwlowR23+EwhiR23)/2 & noisewid=30
 oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)

if ctn lt 1 then begin
 oknoise = where(wl ge (noisecen-noisewid/2) and wl le EwhiR23 ,ctn)

endif

if ct1 lt 1 or ctn lt 1 then print,' -- fail to find proper range of spectra to fit OIII-5007 region' 
if ct1 lt 1 or ctn lt 1 then stop



 wltmp=wl[okha] & a3=alastR23 & sigma3=a3-a3

 ytmp=sm1d[okha] & tmpnoise=varf[okha]
 
                       tmpw= 1./(tmpnoise)^2 ; ;par =[z,Fha,sigma,Fnii,cont]
              nandull=where(finite(tmpw,/nan) OR finite(tmpw,/infinity),badnoisew)
       if badnoisew ge 1 then              tmpw[nandull]=1.e-6 ;; small weighting for NAN, INFINITY values

tagx=' nofit' & tagok=' fit'
if flag2 eq 0 then   tagp=tagx
if flag2 ne 0 then   tagp=tagok

            fit1 = mpcurvefit(wltmp,ytmp,tmpw,a3,sigma3,chisq=chi1f2,dof=dof2,function_name="O3Hb",/noderivative,/quiet,parinfo=parinfo)        
    ;           flimit[fitcount,2]=median(tmpnoise[oknoise])
                            ;
            ;;;;              ;; correct for instrument width
              a3[2] = sqrt(a3[2]^2-(inswid/2.3548)^2)
              sigma3[2] = sigma3[2]/sqrt(2.d*a3[2])
              
                plot,wltmp,ytmp,xra=[EwlowR23,EwhiR23],/xs,title='Hb-[OIII] lines zoom-in'+tagp,charthick=2,charsize=1.2,thick=4,ytitle=' 10^-17 ergs/s/cm2/Ang';,
                oplot,wltmp,tmpnoise,color=150
                if flag2 ne 0 then begin
                oplot,wltmp,fit1,color=50
                oplot,wltmp,ytmp-fit1,color=220
                xyouts,EwlowR23+FWHM_gal,max(fit1)*0.9, ' zspec_OIII= '+strmid(strc(a3[0]),0,5),charthick=2,color=5
               xyouts,EwlowR23+FWHM_gal,min(ytmp)*0.7,'S/N='+strmid(strc((a3[3]/sigma3[3])),0,4)+','+strmid(strc((a3[1]/sigma3[1])),0,4),charthick=4,color=6
               endif
if multinameno eq 0 then begin
  mwrfits,wltmp,Namefxx+'_1D_OIIIHbetaraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D_OIIIHbetaraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D_OIIIHbetabestfit.fits',hdr1d,/create
endif else begin
  mwrfits,wltmp,Namefxx+'_1D-'+mtag+'_OIIIHbetaraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D-'+mtag+'_OIIIHbetaraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D-'+mtag+'_OIIIHbetabestfit.fits',hdr1d,/create
endelse
;;;;
  oplot,[5008.24*(1+zs),5008.24*(1+zs)],[-100,100],linestyle=2     
  oplot,[4960.3*(1+zs),4960.3*(1+zs)],[-100,100],linestyle=2     
  oplot,[4862.69*(1+zs),4862.69*(1+zs)],[-100,100],linestyle=2    


snsee=a3[1]/sigma3[1]

  trigger=where(finite(snsee,/nan) OR finite(snsee,/infinity),csee)


 fitemission[fitcount,parnSII+parnHaNII+0:parnR23+parnSII+parnHaNII-1]=a3
 fitemissionerr[fitcount,parnSII+parnHaNII+0:parnR23+parnSII+parnHaNII-1]=sigma3




     noisecen=5008.24*(1+zs) & noisewid=6.8
     noisewid2=a3[2]  
     if noisewid2 lt 2 OR  noisewid2 gt 30 then noisewid2=6.8

 oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
if ctn ge 1 then begin
flimit[fitcount,5]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fs6717 limit
endif else begin
flimit[fitcount,5]=0;median(tmpnoise[oknoise])*noisewid*sqrt(2.*!pi)   ;; fs6717 limit
endelse 
noisecen=4862.69*(1+zs) & noisewid=6.8
 oknoise = where(wltmp ge (noisecen-noisewid/2) and wltmp le (noisecen+noisewid/2) ,ctn)
if ctn ge 1 then begin
flimit[fitcount,6]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fs6717 limit
endif else begin
flimit[fitcount,6]=0;median(tmpnoise[oknoise])*noisewid*sqrt(2.*!pi)   ;; fs6717 limit
endelse


if csee gt 0 then begin
a4=alastR23_2 & sigma4=a4-a4+1

;  PARINFO Example:
  parinfo = replicate({value:0.D, fixed:0, limited:[0,0],limits:[0.D,0.D]}, n_elements(alastR23_2))
  parinfo(0).limited(0) = 1
   parinfo(0).limits(0)  = zs-0.1D
   parinfo(0).limited(1) = 1
   parinfo(0).limits(1)  = zs+0.1D
   parinfo(1).limited(0) = 1
   parinfo(1).limits(0)  = 1.d-8  ;; flux lower cut 2.d-10= 0 
   parinfo(2).limited(0) = 1
   parinfo(2).limits(0)  = 1.D-8  ;; dispersion lower cut 2.d-10= 0 km/s
   parinfo(2).limited(1) = 1            
   parinfo(2).limits(1)  = 30.d0  ;; dispersion upper cut 1000km/s
 ;  parinfo(3).limited(0) = 1
 ;  parinfo(3).limits(0)  = 1.D-8  ;; flux lower cut 2.d-10= 0 
   parinfo(*).value = alastR23_2  ; [zs, 0.5d, 1,0.003]   
 

 okha = where(wl ge EwlowR23+800 and wl le EwhiR23,ct1)
   noisecen=(EwlowR23+600+EwhiR23)/2 & noisewid=30
 oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)


if ct1 lt 1 or ctn lt 1 then begin
 okha = where(wl ge EwlowR23 and wl le EwhiR23,ct1)
   noisecen=(EwlowR23+EwhiR23)/2 & noisewid=30
 oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
endif
 
if ct1 lt 1 or ctn lt 1 then print,' -- fail to find proper range of spectra to fit ' 
if ct1 lt 1 or ctn lt 1 then stop



 wltmp=wl[okha] & a4=alastR23_2

 ytmp=sm1d[okha] & tmpnoise=varf[okha]
 
             tmpw= 1./(tmpnoise)^2 ; ;par =[z,Fha,sigma,Fnii,cont]
              nandull=where(finite(tmpw,/nan) OR finite(tmpw,/infinity),badnoisew)
       if badnoisew ge 1 then              tmpw[nandull]=1.e-6 ;; small weighting for NAN, INFINITY values
          
              fit1 = mpcurvefit(wltmp,ytmp,tmpw,a4,sigma4,chisq=chi1f2,dof=dof2,function_name="double_O3",/noderivative,/quiet,parinfo=parinfo)        
                                ;     flimit[fitcount,2]=median(tmpnoise[oknoise])
;;;;              ;; correct for instrument width
              a4[2] = sqrt(a4[2]^2-(inswid/2.3548)^2)
              sigma4[2] = sigma4[2]/sqrt(2.d*a4[2])
              
fitemission[fitcount,parnR23+parnSII+parnHaNII+0:parnR23_2+parnR23+parnSII+parnHaNII-1]=a4
 fitemissionerr[fitcount,parnR23+parnSII+parnHaNII+0:parnR23_2+parnR23+parnSII+parnHaNII-1]=sigma4

      endif 
                plot,wltmp,ytmp,xra=[EwlowR23+1100,EwhiR23],/xs,title='[OIII] doublet'+tagp,charthick=2,charsize=1.2,thick=4,ytitle=' 10^-17 ergs/s/cm2/Ang',xtickformat='(F8.1)',xticks=4;,
;;;;
                                oplot,wltmp,tmpnoise,color=150

if multinameno eq 0 then begin
  mwrfits,wltmp,Namefxx+'_1D_OIIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D_OIIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D_OIIIbestfit.fits',hdr1d,/create
endif else begin
  mwrfits,wltmp,Namefxx+'_1D-'+mtag+'_OIIIraw_wave.fits',hdr1d,/create
  mwrfits,ytmp,Namefxx+'_1D-'+mtag+'_OIIIraw.fits',hdr1d,/create
  mwrfits,fit1,Namefxx+'_1D-'+mtag+'_OIIIbestfit.fits',hdr1d,/create
endelse
;;;;
                if flag2 ne 0 then begin
                oplot,wltmp,fit1,color=50

                oplot,wltmp,ytmp-fit1,color=220
  oplot,[5008.24*(1+zs),5008.24*(1+zs)],[-100,100],linestyle=2     
  oplot,[4960.3*(1+zs),4960.3*(1+zs)],[-100,100],linestyle=2
                endif
 noisecen=5008.24*(1+zs) & noisewid=6.8
 oknoise = where(wl ge (noisecen-noisewid/2) and wl le (noisecen+noisewid/2) ,ctn)
    

   noisewid2=a3[2]
     if noisewid2 lt 2 OR  noisewid2 gt 30 then noisewid2=6.8
if ctn ge 1 then flimit[fitcount,5]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fs6717 limit
noisecen=4862.69*(1+zs) & noisewid=6.8
 oknoise = where(wltmp ge (noisecen-noisewid/2) and wltmp le (noisecen+noisewid/2) ,ctn)
if ctn ge 1 then flimit[fitcount,6]=median(varf[oknoise])*noisewid2*sqrt(2.*!pi)   ;; fs6717 limit


if csee gt 0 then begin
          if flag2 ne 0 then begin
             xyouts,EwlowR23+800+FWHM_gal,min(ytmp)*0.7,'S/N='+strmid(strc((a4[1]/sigma4[1])),0,4),charthick=4,color=6
             xyouts,EwlowR23+820+FWHM_gal,max(fit1)*0.8, ' zspec_OIII= '+strmid(strc(a4[0]),0,5),charthick=2,color=5
          endif
endif
 
 
endif   ;;; 


;;Endelse
;; fitcount=fitcount+1
;; print,'     --- counting includes muli-object on   = ',fitcount
jumpskipo3: print,'  -skip o3'
Jump1: print,'  ...  ... ... '
fitcount=fitcount+1

Endfor   ;;ENDFOR-2

;print,xguess
endif else begin
  print,' stopped ---> no object matching the input tables ! Neet to go back to check it '
  print,fitcount
  stop
endelse 



ENDFOR



if keyword_set(ps) then device,/close
if keyword_set(ps) then device,/encapsulated
if keyword_set(ps) then set_plot,'x'





if keyword_set(first) then GOTO, JUMP2

fitemission_tags=StrArr(100)
comments=StrArr(100)
fitemission_tags[0:parnOIId+parnR23+parnSII+parnHaNII-1]=['0=zspec_HaNII','1=flux_Ha','2=Gauss_Sigma_HaNII','3=flux_NII','4=continuum1',$ 
                   '5=zspec_SII','6=flux_SII6717','7=flux_SII6731','8=Gauss_Sigma_SII','9=continuum2',$
                   '10=zspec_OIIIHebta','11=flux_OIII','12=Gauss_Sigma_OIIIHbeta','13=flux_Hbeta','14=continuum3','15=zOII','16=foii_1',$
                     '17=foii_2','18=Gauss_Sigma_OII','19=contin4']

comments[0]=['Flux in unit of 10^-17 ergs/s/cm2; width in unit of Angs=Guassian_sigma(FWHM/2.3548); HaNII are constrained to have the same width and line center']
comments[1]=['fitemissionerr=errors for each parameters from mpcurvefit']
comments[2]=['.flimit= 1 sigma stdeviation in the line fitting vincinity from the stdeviation cube [*,0]=ha,1=n2,2=s2_1,3=s2_2,4=o2,5=03,6=hb']
comments[3]=['Colname1=obj name; COLMOSNAME=obj mask slit ID;colzp=photoz']
comments[4]=['COLFLAG=;; flag 1=fit Ha-NII-SII;  2=fit OIII-Hb; 3=single unknown lines -999 =very likely no lines']
comments[5]=['COLAPECEN & COLAPEWID ;;; spatial center & width on 2D slitlets to extract the 1D spec; COLAPEWID > 5 for spatially extended']
comments[6]=['COLFLAG_SPATIAL ; flag for spatial center/width to extract the 1D spec  ;;; good detection = 1, faint target = 0  ']

;fitcount=fitcount+1

;stop
;;; Save all fitting results in a structure ;;; 
superstruc={superfit,$   ;; structure template name is now assigned to this name
             colname1: strarr(fitcount), $   ;;; obj name e.g., SF_2050 from input table
             colmosname: strarr(fitcount), $  ;; obj mask slit ID name
             colflag: intarr(fitcount), $  ;; flag 1=fit Ha-NII-SII;  2=fit OIII-Hb; 3=single unknown lines -999 =very likely no lines
             colflag_2: intarr(fitcount), $  ;; flag 1=fit Ha-NII-SII;  2=fit OIII-Hb; 3=single unknown lines -999 =very likely no lines
             colzp:fltarr(fitcount),$    ;;; photoz
             colapecen:fltarr(fitcount),$    ;;; spatial center on 2D slitlets to extract the 1D spec
             colapewid:fltarr(fitcount),$    ;;; spatial width on 2D slitlets to extract the 1D spec
       colflag_spatial:intarr(fitcount),$  ;;; flag for spatial center/width to extract the 1D spec  ;;; good detection = 1, faint target = 0 I wont't trust =0
               fitemission: fltarr(fitcount,100),$     ;;; to store the best line fitting parameters
               fitemissionerr: fltarr(fitcount,100),$     ;;; to store the best line fitting parameters
               flimit: fltarr(fitcount,10),$     ;;; to store the best line fitting parameters
                fitemission_tags:StrArr(100), $ ; column tags for line fitting, e.g,zpec_Ha, Gaussian_sigma_Ha,
                comments:StrArr(100) $ ; other comments
}

superstruc.colname1=colname1
superstruc.colmosname=colmosname
superstruc.colflag=colflag
superstruc.colflag_2=colflag_2

superstruc.colzp=colzp
superstruc.colapecen=colapecen
superstruc.colapewid=colapewid
superstruc.colflag_spatial=colflag_spatial
superstruc.fitemission=fitemission
superstruc.fitemissionerr=fitemissionerr
superstruc.fitemission_tags=fitemission_tags
superstruc.comments=comments
superstruc.flimit=flimit


;stop
;spawn,'rm -rf '+path+maskname+'.superfitstruc.fits'

mwrfits,superstruc,path+maskname+'.superfitstruc.fits',/create



;;;;;;;;
zha=reform(superstruc.fitemission[*,0])
zOIII=reform(superstruc.fitemission[*,10])
zha_err=reform(superstruc.fitemissionerr[*,0])
zOIII_err=reform(superstruc.fitemissionerr[*,10])
zOII=reform(superstruc.fitemission[*,15])
zOII_err=reform(superstruc.fitemissionerr[*,15])


;colname1=superstruc.colname1
flag=superstruc.colflag
flag22=superstruc.colflag_2

zphoto=superstruc.colzp
slitids=superstruc.COLMOSNAME

zspec=zha
zspec_err=zha_err

hadex=where(flag eq 6563 OR flag lt 0,ct1)
OIIIdex=where(flag22 eq 5007,ct2)
OIIdex=where(flag eq 3727,ct3)

if ct1 ge 1 then begin
 zspec[hadex]=zha[hadex] 
zspec_err[hadex]=zha_err[hadex] 
endif

if ct2 ge 1 then begin
 zspec[OIIIdex]=zOIII[OIIIdex] 
 zspec_err[OIIIdex]=zOIII_err[OIIIdex]
endif

if ct3 ge 1 then begin
 zspec[OIIdex]=zOII[OIIdex] 
 zspec_err[OIIdex]=zOII_err[OIIdex]

endif

fha=reform(superstruc.fitemission[*,1])
fha_err=reform(superstruc.fitemissionerr[*,1])
fnii=reform(superstruc.fitemission[*,3])
fnii_err=reform(superstruc.fitemissionerr[*,3])
fo3=reform(superstruc.fitemission[*,11])
fo3_err=reform(superstruc.fitemissionerr[*,11])
fhb=reform(superstruc.fitemission[*,13])
fhb_err=reform(superstruc.fitemissionerr[*,13])
fs2_6717=reform(superstruc.fitemission[*,6])
fs2_6717_err=reform(superstruc.fitemissionerr[*,6])
fs2_6731=reform(superstruc.fitemission[*,7])
fs2_6731_err=reform(superstruc.fitemissionerr[*,7])
fo2=(reform(superstruc.fitemission[*,16])+reform(superstruc.fitemission[*,17]))/1.
fo2_err=sqrt((reform(superstruc.fitemissionerr[*,16]))^2+(reform(superstruc.fitemissionerr[*,16]))^2)/1.


SN_n2=fnii/fnii_err
SN_ha=fha/fha_err
SN_o3=fo3/fo3_err
SN_o2=fo2/fo2_err

SN_hb=fhb/fhb_err
SN_s2_6717=fs2_6717/fs2_6717_err
SN_s2_6731=fs2_6731/fs2_6731_err

 
fha_width=reform(superstruc.fitemission[*,2])
fha_width_err=reform(superstruc.fitemissionerr[*,2])
flimit=reform(superstruc.flimit[*,0])
flimit_nii=reform(superstruc.flimit[*,1])

;; thresh_ff1=3
;; conflimit=where(SN_ha lt thresh_ff1,ct)
;; if ct ge 1 then begin
;; fha[conflimit]=flimit[conflimit] & fha_err[conflimit]=fha_err[conflimit]-fha_err[conflimit]-9999.
;; endif
;; conflimit=where(SN_n2 lt thresh_ff1,ct)
;; if ct ge 1 then begin
;; fnii[conflimit]=flimit_nii[conflimit] & fnii_err[conflimit]=fnii_err[conflimit]-fnii_err[conflimit]-9999.
;; endif


fs2_width=reform(superstruc.fitemission[*,8])
fs2_width_err=reform(superstruc.fitemissionerr[*,8])
flimit_s2_1=reform(superstruc.flimit[*,2])
flimit_s2_2=reform(superstruc.flimit[*,3])

;; conflimit=where(SN_s2_6717 lt thresh_ff1,ct)
;; if ct ge 1 then begin
;; fs2_6717[conflimit]=flimit_s2_1[conflimit] & fs2_6717_err[conflimit]=fs2_6717_err[conflimit]-fs2_6717_err[conflimit]-9999.
;; endif
;; conflimit=where(SN_s2_6731 lt thresh_ff1,ct)
;; if ct ge 1 then begin
;; fs2_6731[conflimit]=flimit_s2_2[conflimit] & fs2_6731_err[conflimit]=fs2_6731_err[conflimit]-fs2_6731_err[conflimit]-9999.
;; endif


fo3_width=reform(superstruc.fitemission[*,12])
fo3_width_err=reform(superstruc.fitemissionerr[*,12])
flimit_o3=reform(superstruc.flimit[*,5])
flimit_hb=reform(superstruc.flimit[*,6])

;; conflimit=where(SN_o3 lt thresh_ff1,ct)
;; if ct ge 1 then begin
;; fo3[conflimit]=flimit_o3[conflimit] & fo3_err[conflimit]=fo3_err[conflimit]-fo3_err[conflimit]-9999.
;; endif
;; conflimit=where(SN_hb lt thresh_ff1,ct)
;; if ct ge 1 then begin
;;  fhb[conflimit]=flimit_hb[conflimit] & fhb_err[conflimit]=fhb_err[conflimit]-fhb_err[conflimit]-9999.
;; endif

fo2_width=reform(superstruc.fitemission[*,18])
fo2_width_err=reform(superstruc.fitemissionerr[*,18])
flimit_o2=reform(superstruc.flimit[*,4])

;; conflimit=where(SN_o2 lt thresh_ff1,ct)
;; if ct ge 1 then  begin
;; fo2[conflimit]=flimit_o2[conflimit] & fo2_err[conflimit]=fo2_err[conflimit]-fo2_err[conflimit]-9999.
;; endif



clean=where(finite(zspec_err,/nan) OR finite(zspec_err,/infinity),csee)
if csee gt 1 then zspec_err[clean]=9999

conf=zha-zha+1

conf1=where(flag eq 6563 AND SN_ha gt 3,ct)
if ct ge 1 then conf[conf1]=3  ;;; Confident that it is Halpha

conf11=where(flag eq 3727 AND SN_o2 gt 3,ct)
if ct ge 1 then conf[conf11]=3  ;;; Confident that it is Halpha

conf2=where(flag eq 5007 AND SN_o3 gt 3,ct)
if ct ge 1 then conf[conf2]=3  ;;; Confident that it is OIII

;; conf2=where(flag eq 5007 AND SN_o3 gt 3,ct)
;; if ct ge 1 then zha[conf2]=zOIII[conf2]  ;;; Confident that it is OIII

conf3=where(flag eq 999 AND SN_ha gt 3,ct)
if ct ge 1 then conf[conf3]=2  ;;; single emission line redshift 

conf3=where(flag eq 999 AND SN_o3 gt 3,ct)
if ct ge 1 then conf[conf3]=2  ;;; single emission line redshift 

conf4=where(flag eq -999,ct)
if ct ge 1 then conf[conf4]=1  ;;; no emission line detected
conf5=where(flag eq 0,ct)

if ct ge 1 then begin conf[conf5]=-1
 colname1[conf5]=colname1[conf5]+'no_fit';;; this is a star 
endif

fmtt11='(a25,f10.3,f16.6,f16.6,i5,i10, f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3, f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3, f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3)'
fmtt1='(a20,f10.3,f16.6,f16.6,i5,i10, f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3, f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3)'
cctt='##Columns=Nameobj, zphoto, zspec,zspec_err,conf, Line_flag,fha,fha_err,fnii,fnii_err,fo3,fo3_err,fhb,fhb_err,fs2_6717,fs2_6717_err,fs2_6731,fs2_6731_err,fo2,fo2_err,fha_width,fha_width_err,fo3_width,fo3_width_err,fs2_width,fs2_width_err,fo2_width,fo2_err_width_err'
cct2='## flux unit = 1.e-17 ergs/s/cm2; Line width are Gaussian sigma.  Line width are constrained to be same for NII&Ha, O3&Hb, SII two doublets, respectively,i.e., NII have the same line width as Halpha'
cx='## Line_flag[6563=Ha fit;  5007= OIII fit; 999=single unknown emission lines -999 =very likely no detection 0=star]'
cxx='## conf =[: 3=confident that it is the correct spec-z from Halpha or O3,O2; 2=from single emission line with S/N > 3 assuming it is Ha or O3,O2; 1=no obvious line detection; 0=star ]'
cct3='## Table ascii outputs from IDL structure'
cff='##########'
ccttlim='##Columns=Nameobj, zphoto, zspec,zspec_err,conf, Line_flag,fha,fha_err,fnii,fnii_err,fo3,fo3_err,fhb,fhb_err,fs2_6717,fs2_6717_err,fs2_6731,fs2_6731_err,fo2,fo2_err,fha_width,fha_width_err,fo3_width,fo3_width_err,fs2_width,fs2_width_err,fo2_width,fo2_err_width_err,flimit_ha,flimit_nii,flimit_o3,flimit_hb,flimit_o2,flimit_s2_1,flimit_s2_2'

;stop
;; forprint,colname1,zphoto,  zspec,zspec_err,conf, flag, $ 
 ;;                          fha,fha_err,fnii,fnii_err,fo3,fo3_err, fhb,fhb_err,$
;;                    textout=path+tablename,comment=[cct3,cctt,cct2,cx,cxx], format=fmtt1

forprintb,colname1,zphoto, zspec,zspec_err,conf,flag,$ 
                         fha,fha_err,fnii,fnii_err,fo3,fo3_err,fhb,fhb_err,fs2_6717,fs2_6717_err,fs2_6731,fs2_6731_err,fo2,fo2_err,$
                         fha_width,fha_width_err,fo3_width,fo3_width_err,fs2_width,fs2_width_err,fo2_width,fo2_width_err,$
                   textout=path+tablename,comment=[cct3,cctt,cct2,cx,cxx,cff], format=fmtt1

forprintb,colname1,zphoto, zspec,zspec_err,conf,flag,$ 
                         fha,fha_err,fnii,fnii_err,fo3,fo3_err,fhb,fhb_err,fs2_6717,fs2_6717_err,fs2_6731,fs2_6731_err,fo2,fo2_err,$
                         fha_width,fha_width_err,fo3_width,fo3_width_err,fs2_width,fs2_width_err,fo2_width,fo2_width_err,flimit,flimit_nii,flimit_o3,flimit_hb,flimit_o2,flimit_s2_1,flimit_s2_2,$
                   textout=path+tablename+'.limit',comment=[cct3,ccttlim,cct2,cx,cxx,cff], format=fmtt11
;comment=[cct3,cctt,cct2,cx,cxx,cff]

;spawn,'aquamacs '+path+'simpletab.dat &'

;; fmtt2='(a15, f10.3,f16.6,f16.6,i5,i10, f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3,f14.3)'

;; fmtt3='(f12.3,f12.3,f12.3,f12.3,f12.3,f12.3,f12.3,f12.3,f12.3,f12.3)'
;; forprint,colname1,zphoto, zspec,zspec_err,conf,flag,$ 
;;                          fha,fha_err,fnii,fnii_err,fo3,fo3_err,fhb,fhb_err,fs2_6717,fs2_6717_err,$
;;                    textout=path+'part1.dat',comment=[cct3,cctt,cct2,cx,cxx,cff], format=fmtt2
;; forprint,fs2_6731,fs2_6731_err,fo2,fo2_err,fha_width,fha_width_err,fo3_width,fo3_width_err,fs2_width,fs2_width_err,textout=path+'part2.dat',/nocomment, format=fmtt3
;; stop

;;;tab=mrdfits(path+maskname+'.superfitstruc.fits',1,hdrtab)
;;;help,tab,/str
;;;print,tab.FITEMISSION_TAGS[0:50]
forprint,colname1,colapecen,colapewid,textout=path+maskname+'extract_center_width.dat',comment='name aperture_center     extracting FWHM'   ;; obj flag from input table e.g, 1, -999

;;stop
JUMP2: print,'  end of the program ... '

if keyword_set(first) then  forprint,xguess,yfixornot_arr,textout=path+maskname+'slitcen.dat',comment='object spatial location guess'   ;; obj flag from input table e.g, 1, -999




END
