pro mosrunall_stepall
;;
;;USER INPUT:   EDIT PARAMETERS IN #SECTIONs: 
;;  Purpose: written for post-MOSPY DRP (MOSFIRE) analysis  including telluric correction,
;;  flux calibration,  1D spectra extraction, and emission line
;;       simple Gaussian profile fitting
;;;; HISTORY
;;;;   BUG report to T.-T. Yuan; tiantian.yuan@anu.edu.au
;; last modification: March 12, 2014
;;  

; initialize
!quiet=1
!except=0
astrolib
device, decomposed=0
loadcolors

;; #################--- --- #SECTION: USER INPUT BEGIN --- --- ############################  ;;;; 
;;; EACH OF THE FOLLLOWING STEPS CAN BE RUN SEPERATELY. YOU CAN
;;; COMMENT OUT THE STEPS THAT ARE FINISHED SUCCESSFULLY 
;;;  = 1 => run this step; =0 => skip running this step
step1=1   ;Derive Sensitivity Curve using telluric standard star
step2=0;0    ;Divide science and sigma frames by the Sensitivity Curve
step3=0   ;Derive flux conversion factor using flux standard star AND do flux calibration for science and sigma frames
step4=0   ;;; this is the final step of fitting 1 D spectra emission lines
                    ;;; i recommend running this step seperately,
                    ;;; i.e., after you finish step 1-3.

runbandJ=0     ;;; keywords for the observed band, if your data is in X-band, then Xband=1, band_others=0 
runbandH=0
runbandK=1 

;;;;;;;;
;;; ### datapath = path where you put your DRP processed science data such as COSMOS_2Kz4_K_S01_20169_13236_eps.fits
datapath='/Users/yuantt/work/data/mosfire2017feb13/DRP/140217/cosmos-K3.2/'  
;;; ### rootname =  syntax for the science data, pls follow the exact
;;;                     formatted as below 
rootname='cosmos-K3.2_K_S01_7968_eps'     ;;; could be any science frame rootname.fits 
                                       ;;;; I merely use the header in this frame to set the
                                       ;;;; plotting wavelength range

;;;;; NOTES: STOP and check if filenames contain slitnumber here;
;;;;;;  Because this code was written for very generation of
;;;;; DRP data format, I require the slit number (S01, S02, S03 ...)
;;;;; in the filenames !   Sorry about the inconvenience, but my code is hardwared to trace each target by their
;;;;; slitnumber names.  Can't change them easily. Please add
;;;;; slitnumber after the band tag (e.g, after K_ or H_ or J_)
;;; ======> You can use the example in this code folder
;;;;; rename_*.csh  to add slit number to your filenames..


;;;; ;;;;;;; 
masklist=datapath+'obj.list'   ;;;lists of science object file under your datapath
mask_sdlist=datapath+'sig.list'    ;;; standard deviation (ELECTRONS/S), must in the same unit as the data[use make_stdeve.pro]
;; you can create the lists quickly using commands like,
;; in your datapath directory  
  ;; ls *eps_H_*.fits >inobj_2014feb11.list
  ;;  ls *sd_H_*.fits >inobj_sd_2014feb11.list
  ;;  ls *itime_H_*.fits >inobj_itime_2014feb11.list
 ;;; Run make_stdev.pro or make_stdev_ivar.pro or use your own magic
 ;;; renaming tricks

;;;;; For Telluric Standard Star
temp=9480. ;; ### black body temperature for your telluric star, 9480. for A0V
outname='HIP_34111_cosmos-K3.2_140217_narrow' ;;; ### output file root name for the star, once defined, should use the same name hereafter
;;; ### telpath = path of the telluric data
;telpath='/Users/yuantt/work/data/MOSFIRE/catlg/standards/longslit_131224_telluric_improvedFlat/'
telpath='/Users/yuantt/work/data/mosfire2017feb13/DRP/140217/long2pos_specphot/K/'
;;; ### telname =  syntax for  your telluric data, follow the exact
;;;                format as below
telname='HIP_34111_POSC_NARROW_rectified_K_A-B'
;;; ### outpath= where you put the mosrun IDL *.pro 
outpath='/Users/yuantt/work/data/mosfire2017feb13/mosfit_idl/star1d/'

;;;;; For Flux Standard Star
tempf=9480. ;; ### black body temperature for your flux star, 9480. for A0VHIP43018
magJ=7.86  ;;;2MASS NIR magnitude in Vega magnitudes
magH=7.794
magK=7.747



fluxpath='/Users/yuantt/work/data/mosfire2017feb13/DRP/140217/long2pos_specphot/K/'
fluxrootname ='HIP_34111_POSA_WIDE_rectified_K_A-B'  ; themiya's



;;;;;;  YOU DON"T HAVE TO FEED THE INPUTS FOR THE FOLLOWING
;;;;;;  LINES IF YOU ARE JUST RUNNING STEP 1-3, these are for step 4

pathdat='/Users/yuantt/work/data/mosfire2017feb13/mosfit_idl/mosfit_inputdat/'
intab='cosmos-K3.2.dat'
outputlist='cosmos-K3.2_140217'  ;;;output root name when running fit1dmos_flux
textpos='_K_S'   ;;; unique string character in the science frame filename used to locate the slitlet ID
                    ;;; eg. in Hband data pipeline output =
                    ;;; eps_H_S26_SSF_2705.fits, then 26 is the
                    ;;; slitlet ID, the string left to 26 is used to
                    ;;; extract this number. 
tablename='tab_140217.dat'

;; #################--- ---SECTION USER INPUT  END --- --- ############################  ;;;; 
;; 
;;
;; #################--- ---DO NOT CHANGE THE LINES BELOW --- --- ############################  ;;;; 
;;
;; 
;; +++++++++++++++++  READY TO GO  +++++++++++++++++++   
;;             *              *
;       \        ----0   0--------      /
;               
;           \          0            /
;                    ______  
;                                                         
;
;;; ;; +++++++++++++++++--- -- CALLING SUBROUTINES --- --- +++++++++++++++++++++++++++++++++++  ;;;; 


;;; %%%%%%%%%%%%% STEP 1 BEGIN **********
;;;     Purpose:  reduce telluric star, obtain sensitivy curve.
;;;          ONLY need to run once for each observational night
;;;;;  Now we are ready to call the function to reduce the telluric
;;;;;  star !   
;; note if not K band, then use keyword /bandH, /bandJ ;; /bandY to be
;; added
if step1 eq 1 then begin
 if runbandJ eq 1 then cali_telluric,datapath,rootname,telpath,telname,outpath,outname,temp,/bandJ
 if runbandH eq 1 then cali_telluric,datapath,rootname,telpath,telname,outpath,outname,temp,/bandH,/checkyes
 if runbandK eq 1 then cali_telluric,datapath,rootname,telpath,telname,outpath,outname,temp,/bandK,/checkyes

endif
;;; %%%%%%%%%%%%% STEP 1 END ; YOU CAN COMMENT THE PREVIOUS LINE  OUT IF ALREADY RUN IT ONCE**********
;
;
;;; %%%%%%%%%%%%% STEP 2 BEGIN **********
;;;     Purpose:  divide all science/sigma data by the sensitivity curve
sensname='sens_'+outname+'_1d.fits'   
sensname_wave='sens_'+outname+'_1d_wave.fits'
pathsens=outpath;   ;;;where the sensitivity curve is saved... should be the same as the outpath in step1
if step2 eq 1 then begin
divid_tellu,pathsens,datapath,sensname,sensname_wave,masklist,mask_sdlist
endif
;;; %%%%%%%%%%%%% STEP 2 END ; YOU CAN COMMENT THE PREVIOUS LINE  OUT IF ALREADY RUN IT ONCE**********
;
;
;;; %%%%%%%%%%%%% STEP 3 BEGIN **********
;;;     Purpose:  derive the flux conversion factor using 2mass
;;;     magnitude of the flux standard; will use the sensitivity curve
;;;     derived from STEP1 
bbname= outname+'flux_1D_HdyrmBB.fits'
bbname_wave= outname+'flux_1D_HdyrmBB_wave.fits'
if step3 eq 1 then begin
 if runbandJ eq 1 then moscali_flux,datapath,rootname,fluxpath,fluxrootname,bbname,bbname_wave,magJ,magH,magK,tempf,fluxconvt,/bandJ,/plotyes
 if runbandH eq 1 then moscali_flux,datapath,rootname,fluxpath,fluxrootname,bbname,bbname_wave,magJ,magH,magK,tempf,fluxconvt,/bandH,/plotyes
 if runbandK eq 1 then moscali_flux,pathsens,sensname,sensname_wave,datapath,rootname,fluxpath,fluxrootname,bbname,bbname_wave,magJ,magH,magK,tempf,filestd,fluxconvt,/bandK,/plotyes;,/usespec

objlist=datapath+'obj_t.list'
objlist_std=datapath+'obj_sdev_t.list'
divid_flux,datapath,fluxpath,bbname,bbname_wave,objlist,objlist_std,fluxconvt;,/usespec
endif
;;; %%%%%%%%%%%%% STEP 3 END ; YOU CAN COMMENT THE PREVIOUS LINE  OUT IF ALREADY RUN IT ONCE**********


;;; %%%%%%%%%%%%% STEP 4 BEGIN **********
if step4 eq 1 then begin
;;;        Extract 1D spectrum from flux calibrated 2D, fit emission lines,
;;;            & generate tables
objlist_fc=datapath+'obj_t_fc.list'
objlist_std_fc=datapath+'obj_sdev_t_fc.list'

;;; run it a first time and mannuall input the best spatial position
 if runbandJ eq 1 then begin
fit1dmos_flux,datapath,pathdat,outputlist,intab,objlist_fc,objlist_std_fc,textpos,tablename,/first,/bandJ
;stop
fit1dmos_flux,datapath,pathdat,outputlist,intab,objlist_fc,objlist_std_fc,textpos,tablename,/ps,/bandJ
endif

 if runbandH eq 1 then begin
fit1dmos_flux,datapath,pathdat,outputlist,intab,objlist_fc,objlist_std_fc,textpos,tablename,/first,/bandH
;stop
fit1dmos_flux,datapath,pathdat,outputlist,intab,objlist_fc,objlist_std_fc,textpos,tablename,/ps,/bandH
endif

if runbandK eq 1 then begin
fit1dmos_flux,datapath,pathdat,outputlist,intab,objlist_fc,objlist_std_fc,textpos,tablename,/first,/bandK;,/interact
;stop
fit1dmos_flux,datapath,pathdat,outputlist,intab,objlist_fc,objlist_std_fc,textpos,tablename,/ps,/bandK
endif
spawn,'aquamacs '+datapath+tablename+' &'

endif
;;; %%%%%%%%%%%%% STEP 4 END ; YOU CAN COMMENT THE PREVIOUS LINE  OUT IF ALREADY RUN IT ONCE**********













END





